/*
*    makepass.c
*    91/02/19 john h. dubois iii    john@armory.com
*    94/01/15 Updated to work with long passwords, 
*    to read password(s) from stdin instead of from command line,
*    and to have optional salt arg.
*    94/06/22 Added -h option.
*/
#include <sys/types.h>
#include <stdio.h>
#include <pwd.h>
#include <time.h>
#define SecureWare
#include <sys/security.h>
#include <sys/audit.h>
#include <prot.h>
#include <string.h>

char *MakePass(char *S,char *Salt);
#define BIG
char *Comment = "@(#) makepass 1.1 96/07/03";

main(argc,argv)
int argc;
char *argv[];
{
    char Line[1024];
    char *Newline;
    char Usage[] = "Usage: makepass [-h] [<salt>]\n";

    if (argc > 2) {
	fputs(Usage,stderr);
	exit(1);
    }
    if (argc == 2 && !strcmp("-h",argv[1])) {
	fputs("makepass: encrypt a password.\n",stdout);
	fputs(Usage,stdout);
	fputs(
	    "Passwords are read from the standard input, one per line.\n"\
	    "They are encrypted and printed to the standard output.\n"\
	    "If an argument is given, it is used as the salt.\n",stdout);
	exit(0);
    }

    set_auth_parameters(argc,argv);
    while (fgets(Line,1023,stdin)) {
	if (Newline = strchr(Line,'\n'))
	    *Newline = '\0';
	puts(MakePass(Line,argv[1]));
    }
    exit(0);
}

char *MakePass(S,Salt)
char *S,*Salt;
{
    long salt;
    char saltc[2];
    char *crypt();
    char c;
    int i;
    char *Result;
    int Segs;

    if (!S)
	return((char *)0);
    /* If empty password was given, produce empty output */
    if (!*S)
	return("");
    if (Salt)
	strncpy(saltc,Salt,2);
    else {
	/* Generate random value to use for salt */
	time (&salt);
	salt += getpid();

	/* Make saltc[] be random 6-bit values */
	saltc[0] = salt & 077;
	saltc[1] = (salt >> 6) & 077;
	/* Skip forbidden password characters in salt */
	for (i=0; i<2; i++) {
	    c = saltc[i] + '.';
	    if (c>'9') c += 7;
	    if (c>'Z') c += 6;
	    saltc[i] = c;
	}
    }
#ifdef BIG
    return bigcrypt(S,saltc);
#else
    Segs = (strlen(S) + 7) / 8;
    Result = (char *)malloc(Segs * 11 + 3);
    if (!Result)
	return((char *)0);
    strcpy(Result,crypt(S, saltc));
    /*
    Passwords longer than 8 chars are encoded by encoding each 8 chars
    seperately.  The salt for each segment after the first is the first 2
    characters of the previous encoded password.  The salt is not included in
    the encoded output.  Note: this code sometimes fails.  Secondary salt
    strings that contain forbidden characters appear to be skipped by some
    algorithm.
    */
    for (i = 1; i < Segs; i++) {
	strncpy(saltc,Result + i*11 - 9,2);
	strcpy(Result + 2 + i*11,crypt(S + i*8, saltc) + 2);
    }
    return Result;
#endif
}
